version: '3.3'
services:
  dmagent-mysql:
    image: registry.cn-shenzhen.aliyuncs.com/dmcloudv2/dmagent-mysql:!IMAGE_VERSION
    container_name: mysql
    restart: always
    environment:
      - TZ=Asia/Shanghai
    volumes:
      - /dmagent/mysql/conf:/etc/mysql/conf.d
      - /dmagent/mysql/data:/var/lib/mysql
      - /dmagent/mysql/backup:/dmagent/backup
      - /etc/localtime:/etc/localtime
    networks:
      dmagent:
        aliases:
          - dmagent-mysql
    logging:
      driver: 'json-file'
      options:
        max-file: '3'
        max-size: '50M'


  dmagent-redis:
    image: registry.cn-shenzhen.aliyuncs.com/dmcloudv2/dmagent-redis:!IMAGE_VERSION
    container_name: dmagent-redis
    restart: always
    environment:
      - TZ=Asia/Shanghai
    networks:
      dmagent:
        aliases:
          - dmagent-redis
    logging:
      driver: 'json-file'
      options:
        max-file: '3'
        max-size: '50M'

  dmagent-user-server:
      image: registry.cn-shenzhen.aliyuncs.com/dmcloudv2/dmagent-user-server:!IMAGE_VERSION
      container_name: dmagent-user-server
      restart: always
      privileged: true
      environment:
        - ENV_HOST=!ENV_HOST
        - REDIS_HOST=dmagent-redis
        - DATABASE_HOST=dmagent-mysql
        - TZ=Asia/Shanghai
      ports:
        - 8080:8080
      volumes:
        - /dmagent/logs:/tmp/logs
        - /dmagent/www/resources:/var/data/www/resources
        - /etc/localtime:/etc/localtime
        - /etc/.lcs:/dmagent/license
        - /sbin/dmidecode:/sbin/dmidecode
        - /dev/mem:/dev/mem
      networks:
        dmagent:
          aliases:
            - dmagent-user-server
      depends_on:
        - dmagent-mysql
        - dmagent-redis
      logging:
        driver: 'json-file'
        options:
          max-file: '3'
          max-size: '50M'
      entrypoint: java -Xms256m -Xmx512m -jar /dmagent/server/tenant/tenant-web-server.jar --spring.profiles.active=prod

  dmagent-device-server:
      image: registry.cn-shenzhen.aliyuncs.com/dmcloudv2/dmagent-device-connect:!IMAGE_VERSION
      container_name: dmagent-device-server
      privileged: true
      restart: always
      environment:
        - ENV_HOST=!ENV_HOST
        - REDIS_HOST=dmagent-redis
        - DATABASE_HOST=dmagent-mysql
        - TZ=Asia/Shanghai
      ports:
        - 3002:3002
        - 3005:3005
        - 3333:3333
        - 3300:3300
        - 5000-5032:5000-5032
        - 20006:20006
        - 2555:2555/udp
        - 5061:5061/udp
      volumes:
        - /dmagent/logs:/tmp/logs
        - /dmagent/www/resources:/var/data/www/resources
        - /etc/localtime:/etc/localtime
        - /etc/.lcs:/dmagent/license
        - /sbin/dmidecode:/sbin/dmidecode
        - /dev/mem:/dev/mem
      networks:
        dmagent:
          aliases:
            - dmagent-device-server
      depends_on:
        - dmagent-mysql
        - dmagent-redis
      logging:
        driver: 'json-file'
        options:
          max-file: '3'
          max-size: '50M'
      entrypoint: java -Xms256m -Xmx512m -jar /dmagent/server/device-connect/device-connect-server.jar --spring.profiles.active=prod

  dmagent-web:
    image: registry.cn-shenzhen.aliyuncs.com/dmcloudv2/dmagent-web:!IMAGE_VERSION
    container_name: dmagent-web
    restart: always
    ports:
      - 80:80
      - 443:443
      - 3006:3006
      - 3334:3334
      - 8081-8082:8081-8082
      - 20007:20007
      - 12000:12000
    volumes:
      - /dmagent/www/resources:/var/data/www/resources
      - /dmagent/www/nginx/cert:/etc/nginx/cert
    networks:
      dmagent:
        aliases:
          - dmagent-web
    depends_on:
      - dmagent-user-server
    logging:
      driver: 'json-file'
      options:
        max-file: '3'
        max-size: '50M'

networks:
  dmagent:
    driver: 'bridge'
