version: '3.3'
services:
  nms-mysql:
    image: registry.cn-shenzhen.aliyuncs.com/dmcloudv2/nms-mysql:!IMAGE_VERSION
    container_name: nms-mysql
    restart: always
    environment:
      - TZ=!TZ
    volumes:
      - /dmagent/mysql/conf:/etc/mysql/conf.d
      - /dmagent/mysql/data:/var/lib/mysql
      - /dmagent/mysql/backup:/dmagent/backup
      - /etc/localtime:/etc/localtime
    ports:
      - 3306:3306
    networks:
      nms-network:
        aliases:
          - nms-mysql
    logging:
      driver: 'json-file'
      options:
        max-file: '3'
        max-size: '50M'

  nms-redis:
    image: registry.cn-shenzhen.aliyuncs.com/dmcloudv2/nms-redis:!IMAGE_VERSION
    container_name: nms-redis
    restart: always
    environment:
      - TZ=!TZ
    networks:
      nms-network:
        aliases:
          - nms-redis
    logging:
      driver: 'json-file'
      options:
        max-file: '3'
        max-size: '50M'

  nms-server:
    image: registry.cn-shenzhen.aliyuncs.com/dmcloudv2/nms-server:!IMAGE_VERSION
    container_name: nms-server
    restart: always
    environment:
      - TZ=!TZ
      - ENV_HOST=!ENV_HOST
      - REDIS_HOST=nms-redis
      - DATABASE_HOST=nms-mysql
    ports:
      - 2555:2555/udp
      - 3005:3005
      - 3333:3333
      - 5001-5032:5001-5032
      - 20006:20006
      - 30000-30127:30000-30127
    volumes:
      - /dmagent/logs:/tmp/logs
      - /dmagent/www/resources:/var/data/www/resources
      - /etc/localtime:/etc/localtime
      - /etc/.lcs:/dmagent/license
      - /sbin/dmidecode:/sbin/dmidecode
      - /dev/mem:/dev/mem
#      - /dmagent/server/nms.jar:/dmagent/server/nms.jar
    depends_on:
      - nms-mysql
      - nms-redis
    networks:
      nms-network:
        aliases:
          - nms-server
    logging:
      driver: 'json-file'
      options:
        max-file: '3'
        max-size: '50M'
    entrypoint: java -Xms256m -Xmx512m -jar /dmagent/server/nms.jar --spring.profiles.active=prod

  nms-web:
    image: registry.cn-shenzhen.aliyuncs.com/dmcloudv2/nms-web:!IMAGE_VERSION
    container_name: nms-web
    restart: always
    ports:
      - 80:80
      - 443:443
      - 3006:3006
      - 3334:3334
      - 20007:20007
    volumes:
      - /dmagent/www/resources:/var/data/www/resources
#      - /dmagent/www/site:/dmagent/www/site
#      - /dmagent/www/dist:/dmagent/www/dist
    networks:
      nms-network:
        aliases:
          - nms-web
    depends_on:
      - nms-server
    logging:
      driver: 'json-file'
      options:
        max-file: '3'
        max-size: '50M'

networks:
  nms-network:
    driver: 'bridge'