#!/bin/bash
FILE_CONF="./nms_conf.xml"

# system parameter names[11]
PARAM_NAME=( "ENV_HOST"
             "REDIS_HOST"
             "REDIS_PASSWD"
             "DATABASE_HOST"
             "DATABASE_NAME"
             "DATABASE_USERNAME"
             "DATABASE_PASSWD"
             "TZ"
             "VERSION"
             "TYPE"
           );
PARAM_NAME2=""

# system parameter desc
PARAM_DESC=( 
             "external{SPACE}ip{SPACE}address,0.0.0.0{SPACE}-{SPACE}default"
             "redis{SPACE}url/ip"
             "redis{SPACE}password"
             "mysql{SPACE}url/ip"
             "mysql{SPACE}database"
             "mysql{SPACE}user"
             "mysql{SPACE}password"
             "Asia/Shanghai"
             "version"
             "default/singleton/old"
           );
PARAM_DESC2=""

# system parameter values
PARAM_VAL=""
PARAM_VAL2="";

# system parameter count
PARAM_NUM=0

# -------------------------------------------------
# convert multiple param_name into array
# -------------------------------------------------
convert_param_name_to_array()
{
    local my_item=""
    local my_index=0

    unset PARAM_NAME2
    for my_item in ${PARAM_NAME[*]}
    do
        PARAM_NAME2[$my_index]=$my_item
        my_index=$(($my_index+1))
    done

    PARAM_NUM=$my_index
}

# -------------------------------------------------
# convert multiple param_desc into array
# -------------------------------------------------
convert_param_desc_to_array()
{
    local my_item=""
    local my_index=0

    unset PARAM_DESC2
    for my_item in ${PARAM_DESC[*]}
    do
        PARAM_DESC2[$my_index]=`echo $my_item | sed "s/{SPACE}/\ /g"`
        ### echo "PARAM_DESC2[$my_index]="${PARAM_DESC2[$my_index]}
        my_index=$(($my_index+1))
    done
}

# -------------------------------------------------
# get one param value from xml file:
#     <param name="param01" value="1"/>
# param_name MUST be unique one
# return: PARAM_VAL2
# -------------------------------------------------
get_xml_param_val()
{
    local my_file=$1
    local my_param=$2

    PARAM_VAL=`cat $my_file | grep $my_param | awk '{print $3}' | cut -f 2 -d '"'`
}

# -------------------------------------------------
# set one param value from xml file:
#     <param name="param01" value="1"/>
# param_name MUST be unique one
# return: PARAM_VAL2
# -------------------------------------------------
set_xml_param_val()
{
    local my_file=$1
    local my_param=$2
    local my_value=$3

    sed -i "/<.*name=\"$my_param\"/ s/value=\".*\"/value=\"$my_value\"/" $my_file
    if [ $? -ne 0 ]; then
    {
        printf "ERROR! set xml param %s fail, value=%s! \n" $my_param $my_value
        return 1
    }
    fi

    return 0
}

# -------------------------------------------------
# load all param in PARAM_NAME[]
# -------------------------------------------------
load_xml_param_list()
{
    local my_index=0
    local my_param=""

    unset PARAM_VAL2

    ### load all param
    for my_param in ${PARAM_NAME2[*]}
    do
        ### get param value
        get_xml_param_val $FILE_CONF $my_param

        ### set param value array
        if [ -z $PARAM_VAL ]; then
            printf "ERROR! Unknown param %s, load fail! \n" $my_param
            printf "Please delete the invalid file: %s, then configure again! \n" $FILE_CONF
            # rm -f $FILE_CONF
            exit 1
        fi

        PARAM_VAL2[ $my_index ]="$PARAM_VAL"

        ### echo "PARAM_VAL2[ $my_index ]="${PARAM_VAL2[ $my_index ]}

        my_index=$(($my_index+1))
    done
}

# -------------------------------------------------
# lookup all param in PARAM_NAME[]
# -------------------------------------------------
show_xml_param_list()
{
    local my_index=0
    local my_param=""
    local my_num=0

    ### print list title
    printf "\n%-4s %-24s : %-32s %s \n" "No." "Name" "Value" "Desc"
    printf "%s \n" "---------------------------------------------------------------------------"

    ### print list content
    for my_param in ${PARAM_NAME2[*]}
    do
        ### print param info
        my_num=$(($my_index+1))
        printf "%-4d %-24s : %-32s %s \n" $my_num $my_param ${PARAM_VAL2[$my_index]} "${PARAM_DESC2[$my_index]}"
        my_index=$(($my_index+1))
    done
}

# -------------------------------------------------
# update all param in PARAM_NAME[]
# -------------------------------------------------
update_xml_param_val()
{
    local my_num=0
    local my_index=0
    local my_value=""
    local my_flag=false

    while true;
    do
    {
            if [ "$AUTO_MODE" == "auto" ];then
                printf "To change value, please enter parameter No.( 0 - quit ): 0 \n"
                        break;
            fi

                ### need confirm            
        printf "To change value, please enter parameter No.( 0 - quit ): "
        read my_num

        ### check is null?
        if [ -z $my_num ]; then
            break;
        elif [ "$my_num" = "" ]; then
            break;
        fi

        ### check is valid number?
        my_flag=`awk 'BEGIN { if (match(ARGV[1],"^[0-9]+$") != 0) print "true"; else print "false" }' $my_num`
        if [ $my_flag == "false" ]; then
            continue;
        fi

        ### check is 0-quit?
        if [ $my_num -eq 0 ]; then
            break;
        fi

        ### check is valid index?
        if [ $my_num -gt $PARAM_NUM ]; then
            continue;
        fi

        my_index=$(($my_num-1))

        printf "Enter new value of parameter %s: " ${PARAM_NAME2[$my_index]}
        read my_value

        if [ ! -z $my_value ]; then
            PARAM_VAL2[$my_index]=$my_value
        fi

        show_xml_param_list
    }
    done
}

# -------------------------------------------------
# save all param in PARAM_NAME[]
# -------------------------------------------------
save_xml_param_val()
{
    local my_index=0
    local my_flag=""

    my_index=0

    while true;
    do
    {
        if [ $my_index -eq $PARAM_NUM ]; then
            break;
        fi

        set_xml_param_val $FILE_CONF ${PARAM_NAME2[$my_index]} ${PARAM_VAL2[$my_index]}

        my_index=$(($my_index+1))
    }
    done

    ### convert to unix format
    dos2unix $FILE_CONF

    printf "*** SUCCESS! Updated system conf file: %s \n" $FILE_CONF
}


if [[ $0 =~ ^\/.* ]]
then
  script=$0
else
  script=$(pwd)/$0
fi

script=`readlink -f ${script}`
script_path=${script%/*}
real_path=$(readlink -f ${script_path})

function echo_usage() {
    echo_green "[Usage]: ./nms.sh install"
    echo_green "[Example]: ./nms.sh install"
    echo_green "[Notes]: Please input y or yes to access all operations."
    exit 1
}

function echo_green() {
    echo -e "\033[32m$@\033[0m"
}

function echo_red() {
    echo -e "\033[31m$@\033[0m"
}

function echo_warn() {
    echo -e "\033[33m$@\033[0m"
}

is_root(){
    ROLE=`id | awk '{print $1}' | sed -e 's/=/ /' -e 's/(/ /' -e 's/)/ /' |awk '{print $3}'`
    if [[ "${ROLE}" != "root" ]]; then
        return 1
    fi
    return 0
}

install(){
    if [[ ${PARAM_VAL2[0]} == "" ]]; then
        echo_red "[ERROR]: Invalid parameters"
        echo_usage
        exit 1
    else
        echo "[INFO]: Begin to install NMS into ${PARAM_VAL2[0]}"
    fi
    echo_green "[INFO]: Is operation permitted?"
    is_root
    if [[ $? -ne 0 ]]; then
        echo_red "[ERROR]: Operation not permitted! Please run scripts by root ..."
        exit 1
    fi

    echo_green "[INFO]: Is docker engine installed?"
    docker --version
    if [[ $? != 0 ]];then
        echo_red "[ERROR]: docker engine didn't installed, Please install docker engine first!"
        echo_green "[INFO]: Please try to install from https://docs.docker.com/engine/install/"
        exit 1
    fi

    echo_green "[INFO]: Is docker-compose installed?"
    docker-compose --version
    if [[ $? != 0 ]]; then
        echo_red "[ERROR]: docker-compose didn't installed, Please try to install it!"
        echo_green "[INFO]: Please try to install from https://docs.docker.com/compose/install/"
        exit 1
    fi

    echo_green "[INFO]: login docker repository, ***if you haven't password please contact our support department!***"
    docker login --username=support@dinstar registry.cn-shenzhen.aliyuncs.com
    if [[ $? != 0 ]]; then
        echo_red "[ERROR]: login docker repository failed, ***if you haven't password please contact our support department!***"
        echo_red "[ERROR]: Existed."
        exit 1
    fi

    echo_green "[INFO]: Installing Configuration Files."
    if [[ ! -d /dmagent ]]; then
        mkdir -p /dmagent
        chmod 755 /dmagent
    fi
# -------------------------------------------------
# select docker-compose
# -------------------------------------------------
    if [[ ${PARAM_VAL2[0]} == "singleton" ]]; then
        apply_singleton_yml
    elif [[ ${PARAM_VAL2[0]} == "old" ]]; then
        apply_deprecated_yml
    else
        apply_new_yml
    fi

    if [[ ! -d /dmagent/www/resources/templates ]]; then
        mkdir -p /dmagent/www/resources
        echo_green "[INFO]: Downloading template files..."
        if [[ ${PARAM_VAL2[0]} == "old" ]]; then
                wget --no-cache http://dm-agent.oss-cn-chengdu.aliyuncs.com/installer/templates.tar.gz -O /dmagent/www/resources/templates.tar.gz
            else
                wget --no-cache http://dm-agent.oss-cn-chengdu.aliyuncs.com/installer/templates_new.tar.gz -O /dmagent/www/resources/templates.tar.gz
        fi
        tar xvzf /dmagent/www/resources/templates.tar.gz  -C /dmagent/www/resources/
        rm -fr /dmagent/www/resources/templates.tar.gz
    fi

    if [[ ! -d /dmagent/www/resources/IPPhone ]]; then
        mkdir -p /dmagent/www/resources
        echo_green "[INFO]: Downloading device config files..."
        wget --no-cache http://dm-agent.oss-cn-chengdu.aliyuncs.com/installer/deviceConfig.tar.gz -O /dmagent/www/resources/deviceConfig.tar.gz
        tar xvzf /dmagent/www/resources/deviceConfig.tar.gz  -C /dmagent/www/resources/
        rm -fr /dmagent/www/resources/deviceConfig.tar.gz
    fi

    echo_green "[INFO]: Downloading docker images and try to startup it..."
    startup 120
    if [[ $? -ne 0 ]];then
        echo_red "[ERROR]: Installing NMS Failed."
        exit 1
    fi

    echo_green "[INFO]: Installing NMS finished."
    exit 0
}

apply_deprecated_yml(){
    cp ${real_path}/docker-compose-deprecated.yml.tpl /dmagent/docker-compose.yml
    sed -i "s|!ENV_HOST|${PARAM_VAL2[0]}|g" /dmagent/docker-compose.yml
    sed -i "s|!IMAGE_VERSION|${PARAM_VAL2[8]}|g" /dmagent/docker-compose.yml
    sed -i "s|!TZ|${PARAM_VAL2[7]}|g" /dmagent/docker-compose.yml
}

apply_new_yml(){
    cp ${real_path}/docker-compose.yml.tpl /dmagent/docker-compose.yml
    sed -i "s|!ENV_HOST|${PARAM_VAL2[0]}|g" /dmagent/docker-compose.yml
    sed -i "s|!IMAGE_VERSION|${PARAM_VAL2[8]}|g" /dmagent/docker-compose.yml
    sed -i "s|!TZ|${PARAM_VAL2[7]}|g" /dmagent/docker-compose.yml
}

apply_singleton_yml(){
    cp ${real_path}/docker-compose-singleton.yml.tpl /dmagent/docker-compose.yml
    sed -i "s|!ENV_HOST|${PARAM_VAL2[0]}|g" /dmagent/docker-compose.yml
    sed -i "s|!IMAGE_VERSION|${PARAM_VAL2[8]}|g" /dmagent/docker-compose.yml
    sed -i "s|!REDIS_HOST|${PARAM_VAL2[1]}|g" /dmagent/docker-compose.yml
    sed -i "s|!REDIS_PASSWD|${PARAM_VAL2[2]}|g" /dmagent/docker-compose.yml
    sed -i "s|!DATABASE_NAME|${PARAM_VAL2[3]}|g" /dmagent/docker-compose.yml
    sed -i "s|!DATABASE_HOST|${PARAM_VAL2[4]}|g" /dmagent/docker-compose.yml
    sed -i "s|!DATABASE_USERNAME|${PARAM_VAL2[5]}|g" /dmagent/docker-compose.yml
    sed -i "s|!DATABASE_PASSWD|${PARAM_VAL2[6]}|g" /dmagent/docker-compose.yml
    sed -i "s|!TZ|${PARAM_VAL2[7]}|g" /dmagent/docker-compose.yml
}

startup(){
    delay=$1
    if [[ "${delay}" == "" ]]; then
        delay=10
    fi

    is_root
    if [[ $? -ne 0 ]]; then
        echo_red "[ERROR]: Operation not permitted! Please run scripts by root ..."
        exit 1
    fi
    docker-compose -f /dmagent/docker-compose.yml up -d
    echo_green "[INFO]: Waiting for docker container starting..."
        if [[ ${PARAM_VAL2[0]} == "singleton" ]]; then
            start_singleton
        elif [[ ${PARAM_VAL2[0]} == "old" ]]; then
            start_deprecated
        else
            start_new
        fi
}

stop(){
    delay=$1
    if [[ "${delay}" == "" ]]; then
      delay=3
    fi
    is_root
    if [[ $? -ne 0 ]]; then
        echo_red "[ERROR]: Operation not permitted! Please run scripts by root ..."
        exit 1
    fi
    echo_warn "[INFO]: NM service will be exist..."
    docker-compose -f /dmagent/docker-compose.yml down
    sleep ${delay}
    echo_green "[INFO]: NM Service has been Existed."
}

restart(){
    stop
    startup
}

status(){
    is_root
    if [[ $? -ne 0 ]]; then
        echo_red "[ERROR]: Operation not permitted! Please run scripts by root ..."
        exit 1
    fi
    docker ps
}

start_new(){
    sleep 10
    step=1
    value=0
    while [[ 1 ]]
    do
        ret1=`docker inspect --format '{{.State.Running}}' nms-mysql`
        ret2=`docker inspect --format '{{.State.Running}}' nms-redis`
        ret3=`docker inspect --format '{{.State.Running}}' nms-server`
        ret4=`docker inspect --format '{{.State.Running}}' nms-web`
        if [[ "${ret1}" == "true" ]] && [[ "${ret2}" == "true" ]] && [[ "${ret3}" == "true" ]] && [[ "${ret4}" == "true" ]]; then
            echo_green "[INFO]: Docker container has been started, wait for NMS starting..."
            sleep ${delay}
            echo_green "[INFO]: NM Service Started."
            echo_green "[INFO]: NM Service Start Success! Please visit http://ip:80 through Google Chrome browser."
            return 0
        else
            value=`expr ${value} + ${step}`
            if [[ ${value} -eq 10 ]]; then
                echo_red "[ERROR]: Starting NM Service failed.."
                stop
                echo_green "[INFO]: Please contact our support department for help."
                break
            fi
            echo_green "[INFO]: Waiting..."
            sleep 2
        fi
    done
    return 1
}

start_deprecated(){
    sleep 10
    step=1
    value=0
    while [[ 1 ]]
    do
        ret1=`docker inspect --format '{{.State.Running}}' dmagent-web`
        ret2=`docker inspect --format '{{.State.Running}}' dmagent-device-server`
        ret3=`docker inspect --format '{{.State.Running}}' dmagent-user-server`
        ret4=`docker inspect --format '{{.State.Running}}' mysql`
        ret5=`docker inspect --format '{{.State.Running}}' dmagent-redis`
        if [[ "${ret1}" == "true" ]] && [[ "${ret2}" == "true" ]] && [[ "${ret3}" == "true" ]] && [[ "${ret4}" == "true" ]] && [[ "${ret5}" == "true" ]]; then
            echo_green "[INFO]: Docker container has been started, wait for NMS starting..."
            sleep ${delay}
            echo_green "[INFO]: NM Service Started."
            echo_green "[INFO]: NM Service Start Success! Please visit http://ip:80 through Google Chrome browser."
            return 0
        else
            value=`expr ${value} + ${step}`
            if [[ ${value} -eq 10 ]]; then
                echo_red "[ERROR]: Starting NM Service failed.."
                stop
                echo_green "[INFO]: Please contact our support department for help."
                break
            fi
            echo_green "[INFO]: Waiting..."
            sleep 2
        fi
    done
    return 1
}

start_singleton(){
    sleep 10
    step=1
    value=0
    while [[ 1 ]]
    do
        ret1=`docker inspect --format '{{.State.Running}}' nms-singleton-server`
        if [[ "${ret1}" == "true" ]]; then
            echo_green "[INFO]: Docker container has been started, wait for NMS starting..."
            sleep ${delay}
            echo_green "[INFO]: NM Service Started."
            echo_green "[INFO]: NM Service Start Success! Please visit http://ip:80 through Google Chrome browser."
            return 0
        else
            value=`expr ${value} + ${step}`
            if [[ ${value} -eq 10 ]]; then
                echo_red "[ERROR]: Starting NM Service failed.."
                stop
                echo_green "[INFO]: Please contact our support department for help."
                break
            fi
            echo_green "[INFO]: Waiting..."
            sleep 2
        fi
    done
    return 1
}

###### main() ######

convert_param_name_to_array
convert_param_desc_to_array
load_xml_param_list
printf "\nPlease confirm the nms parameters:\n"
show_xml_param_list
update_xml_param_val
save_xml_param_val
case "$1" in
  "install")
    install
    ;;
  "update")
    echo_green "[INFO]: Not Support, Please waiting for updating."
    ;;
  "startup")
    startup
    ;;
  "stop")
    stop
    ;;
  "restart")
    restart
    ;;
  "status")
    status
    ;;
  *)
    echo_usage
    ;;
esac

